@@ -83,6 +83,48 @@ enum
 
 static GParamSpec *properties [LAST_PROP];
 
+static void
+dummy_callback (NautilusFile *file,
+                gpointer      callback_data)
+{
+  /* No-op. Just a tag for nautilus_file_cancel_call_when_ready() to match. */
+}
+
+static void
+on_file_changed (NautilusGtkSidebarRow *self)
+{
+  g_return_if_fail (NAUTILUS_IS_GTK_SIDEBAR_ROW (self));
+  g_return_if_fail (NAUTILUS_IS_FILE (self->file));
+  if (nautilus_file_is_gone (self->file))
+    {
+      g_signal_handlers_disconnect_by_func (self->file, on_file_changed, self);
+      nautilus_file_cancel_call_when_ready (self->file, dummy_callback, self);
+      g_clear_object (&self->file);
+    }
+}
+
+static void
+ensure_connected_file (NautilusGtkSidebarRow *self)
+{
+  if (self->file != NULL)
+    return;
+
+  if (self->uri != NULL)
+    {
+      g_autoptr (NautilusFile) file = nautilus_file_get_by_uri (self->uri);
+
+      /* Self-owned file may be marked as gone, if we are rebuilding the sidebar
+       * in response to that file being unmounted. Don't keep it alive. */
+      if (file == NULL || nautilus_file_is_gone (file))
+        return;
+
+      self->file = g_steal_pointer (&file);
+      nautilus_file_call_when_ready (self->file, NAUTILUS_FILE_ATTRIBUTE_MOUNT, dummy_callback, self);
+      g_signal_connect_object (self->file, "changed",
+                               G_CALLBACK (on_file_changed), self, G_CONNECT_SWAPPED);
+    }
+}
+
 static void
 cloud_row_update (NautilusGtkSidebarRow *self)
 {
@@ -179,6 +221,7 @@ nautilus_gtk_sidebar_row_get_property (GObject    *object,
       break;
 
     case PROP_NAUTILUS_FILE:
+      ensure_connected_file (self);
       g_value_set_object (value, self->file);
       break;
 
@@ -245,12 +288,12 @@ nautilus_gtk_sidebar_row_set_property (GObject      *object,
           {
             self->end_icon = G_ICON (g_object_ref (object));
             gtk_image_set_from_gicon (GTK_IMAGE (self->end_icon_widget), self->end_icon);
-            gtk_widget_show (self->end_icon_widget);
+            gtk_widget_set_visible (self->end_icon_widget, TRUE);
           }
         else
           {
             gtk_image_clear (GTK_IMAGE (self->end_icon_widget));
-            gtk_widget_hide (self->end_icon_widget);
+            gtk_widget_set_visible (self->end_icon_widget, FALSE);
           }
         break;
       }
@@ -276,9 +319,9 @@ nautilus_gtk_sidebar_row_set_property (GObject      *object,
     case PROP_EJECTABLE:
       self->ejectable = g_value_get_boolean (value);
       if (self->ejectable)
-        gtk_widget_show (self->eject_button);
+        gtk_widget_set_visible (self->eject_button, TRUE);
       else
-        gtk_widget_hide (self->eject_button);
+        gtk_widget_set_visible (self->eject_button, FALSE);
       break;
 
     case PROP_ORDER_INDEX:
@@ -301,12 +344,7 @@ nautilus_gtk_sidebar_row_set_property (GObject      *object,
     case PROP_URI:
       g_free (self->uri);
       self->uri = g_strdup (g_value_get_string (value));
-      if (self->uri != NULL)
-        {
-          self->file = nautilus_file_get_by_uri (self->uri);
-          if (self->file != NULL)
-            nautilus_file_call_when_ready (self->file, NAUTILUS_FILE_ATTRIBUTE_MOUNT, NULL, NULL);
-        }
+      ensure_connected_file (self);
       break;
 
     case PROP_DRIVE:
@@ -383,13 +421,13 @@ on_child_revealed (GObject    *self,
   * still allocate the paddings, even if the revealer is not revealed, and
   * therefore the row will be still somewhat visible. */
   if (!gtk_revealer_get_reveal_child (GTK_REVEALER (self)))
-    gtk_widget_hide (GTK_WIDGET (NAUTILUS_GTK_SIDEBAR_ROW (user_data)));
+    gtk_widget_set_visible (GTK_WIDGET (NAUTILUS_GTK_SIDEBAR_ROW (user_data)), FALSE);
 }
 
 void
 nautilus_gtk_sidebar_row_reveal (NautilusGtkSidebarRow *self)
 {
-  gtk_widget_show (GTK_WIDGET (self));
+  gtk_widget_set_visible (GTK_WIDGET (self), TRUE);
   gtk_revealer_set_reveal_child (GTK_REVEALER (self->revealer), TRUE);
 }
 
