@@ -21,7 +21,6 @@
 
 #include "nautilus-directory-private.h"
 
-#include <eel/eel-glib-extensions.h>
 #include <eel/eel-string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -66,7 +65,7 @@ static NautilusDirectory *nautilus_directory_new (GFile *location);
 static void               set_directory_location (NautilusDirectory *directory,
                                                   GFile             *location);
 
-G_DEFINE_TYPE (NautilusDirectory, nautilus_directory, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (NautilusDirectory, nautilus_directory, G_TYPE_OBJECT);
 
 static gboolean
 real_contains_file (NautilusDirectory *self,
@@ -333,14 +332,13 @@ nautilus_directory_class_init (NautilusDirectoryClass *klass)
                              G_TYPE_FILE,
                              G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
 
-    g_type_class_add_private (klass, sizeof (NautilusDirectoryDetails));
     g_object_class_install_properties (object_class, NUM_PROPERTIES, properties);
 }
 
 static void
 nautilus_directory_init (NautilusDirectory *directory)
 {
-    directory->details = G_TYPE_INSTANCE_GET_PRIVATE ((directory), NAUTILUS_TYPE_DIRECTORY, NautilusDirectoryDetails);
+    directory->details = nautilus_directory_get_instance_private (directory);
     directory->details->file_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                            g_free, NULL);
     directory->details->high_priority_queue = nautilus_file_queue_new ();
@@ -436,6 +434,11 @@ emit_change_signals_for_all_files_in_all_directories (void)
     GList *dirs, *l;
     NautilusDirectory *directory;
 
+    if (directories == NULL)
+    {
+        return;
+    }
+
     dirs = NULL;
     g_hash_table_foreach (directories,
                           collect_all_directories,
@@ -1855,7 +1858,7 @@ nautilus_directory_match_pattern (NautilusDirectory *directory,
         file = NAUTILUS_FILE (l->data);
         name = nautilus_file_get_display_name (file);
 
-        if (g_pattern_match_string (spec, name))
+        if (g_pattern_spec_match_string (spec, name))
         {
             ret = g_list_prepend (ret, nautilus_file_ref (file));
         }
