@@ -21,28 +21,20 @@
 #pragma once
 
 #include <glib-object.h>
+#include <gio/gio.h>
 
 #define NAUTILUS_FDO_DBUS_IFACE "org.freedesktop.FileManager1"
 #define NAUTILUS_FDO_DBUS_NAME  "org.freedesktop.FileManager1"
 #define NAUTILUS_FDO_DBUS_PATH  "/org/freedesktop/FileManager1"
 
 #define NAUTILUS_TYPE_FREEDESKTOP_DBUS nautilus_freedesktop_dbus_get_type()
-#define NAUTILUS_FREEDESKTOP_DBUS(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST ((obj), NAUTILUS_TYPE_FREEDESKTOP_DBUS, NautilusFreedesktopDBus))
-#define NAUTILUS_FREEDESKTOP_DBUS_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_FREEDESKTOP_DBUS, NautilusFreedesktopDBusClass))
-#define NAUTILUS_IS_FREEDESKTOP_DBUS(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NAUTILUS_TYPE_FREEDESKTOP_DBUS))
-#define NAUTILUS_IS_FREEDESKTOP_DBUS_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_FREEDESKTOP_DBUS))
-#define NAUTILUS_FREEDESKTOP_DBUS_GET_CLASS(obj) \
-  (G_TYPE_INSTANCE_GET_CLASS ((obj), NAUTILUS_TYPE_FREEDESKTOP_DBUS, NautilusFreedesktopDBusClass))
 
-typedef struct _NautilusFreedesktopDBus NautilusFreedesktopDBus;
-typedef struct _NautilusFreedesktopDBusClass NautilusFreedesktopDBusClass;
+G_DECLARE_FINAL_TYPE (NautilusFreedesktopDBus, nautilus_freedesktop_dbus, NAUTILUS, FREEDESKTOP_DBUS, GObject);
 
-GType nautilus_freedesktop_dbus_get_type (void);
-NautilusFreedesktopDBus * nautilus_freedesktop_dbus_new (void);
+NautilusFreedesktopDBus * nautilus_freedesktop_dbus_new (GDBusConnection *connection);
+
+gboolean nautilus_freedesktop_dbus_register (NautilusFreedesktopDBus *fdb, GError **error);
+void nautilus_freedesktop_dbus_unregister (NautilusFreedesktopDBus *fdb);
 
 void nautilus_freedesktop_dbus_set_open_locations (NautilusFreedesktopDBus *fdb, const gchar **locations);
 
