@@ -367,10 +367,8 @@ on_operations_icon_draw (GtkDrawingArea            *drawing_area,
     gboolean all_cancelled;
     GdkRGBA background;
     GdkRGBA foreground;
-    GtkStyleContext *style_context;
 
-    style_context = gtk_widget_get_style_context (widget);
-    gtk_style_context_get_color (style_context, &foreground);
+    gtk_widget_get_color (widget, &foreground);
     background = foreground;
     background.alpha *= 0.3;
 
@@ -469,7 +467,7 @@ operations_list_create_widget (GObject  *item,
     GtkWidget *widget;
 
     widget = nautilus_progress_info_widget_new (info);
-    gtk_widget_show (widget);
+    gtk_widget_set_visible (widget, TRUE);
 
     return widget;
 }
