@@ -76,10 +76,6 @@ enum
 
 static guint signals[LAST_SIGNAL];
 
-static void entry_activate_cb (GtkWidget           *entry,
-                               NautilusQueryEditor *editor);
-static void entry_changed_cb (GtkWidget           *entry,
-                              NautilusQueryEditor *editor);
 static void nautilus_query_editor_changed (NautilusQueryEditor *editor);
 
 G_DEFINE_TYPE (NautilusQueryEditor, nautilus_query_editor, GTK_TYPE_WIDGET);
@@ -626,10 +622,11 @@ nautilus_query_editor_init (NautilusQueryEditor *editor)
     gtk_widget_set_parent (image, GTK_WIDGET (editor));
 
     editor->tags_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-    gtk_widget_hide (editor->tags_box);
+    gtk_widget_set_visible (editor->tags_box, FALSE);
     gtk_widget_set_parent (editor->tags_box, GTK_WIDGET (editor));
 
     editor->text = gtk_text_new ();
+    gtk_text_set_placeholder_text (GTK_TEXT (editor->text), _("Search files and folders"));
     gtk_widget_set_hexpand (editor->text, TRUE);
     gtk_widget_set_parent (editor->text, GTK_WIDGET (editor));
 
@@ -788,17 +785,6 @@ nautilus_query_editor_set_query (NautilusQueryEditor *self,
     self->change_frozen = FALSE;
 }
 
-void
-nautilus_query_editor_set_text (NautilusQueryEditor *self,
-                                const gchar         *text)
-{
-    g_return_if_fail (NAUTILUS_IS_QUERY_EDITOR (self));
-    g_return_if_fail (text != NULL);
-
-    /* The handler of the entry will take care of everything */
-    gtk_editable_set_text (GTK_EDITABLE (self->text), text);
-}
-
 static gboolean
 nautilus_gtk_search_entry_is_keynav_event (guint           keyval,
                                            GdkModifierType state)
